@@ -47,6 +47,10 @@
 #define ASI_STYLE_NAMES 1
 #endif
 
+#include "hpi_internal.h"
+#include "hpimsginit.h"
+#include "hpioctl.h"
+
 #ifndef KERNEL_ALSA_BUILD
 /* building in ALSA source tree */
 #include "adriver.h"
@@ -71,8 +75,6 @@
 #include <sound/tlv.h>
 #include <sound/hwdep.h>
 
-#include "hpi.h"
-#include "hpioctl.h"
 
 MODULE_LICENSE("GPL");
 MODULE_AUTHOR("AudioScience Inc. <support@audioscience.com>");
@@ -155,6 +157,8 @@ struct snd_card_asihpi {
 	int support_mmap;
 	int support_grouping;
 	int support_mrx;
+	u16 in_max_chans;
+	u16 out_max_chans;
 };
 
 /* Per stream data */
@@ -185,14 +189,13 @@ static u16 HPI_StreamHostBufferAttach(
 {
 	struct hpi_message hm;
 	struct hpi_response hr;
+	unsigned int obj = HPI_HandleObject(hStream);
 
 	if (!hStream)
 		return HPI_ERROR_INVALID_OBJ;
-	if (HPI_HandleObject(hStream) ==  HPI_OBJ_OSTREAM)
-		HPI_InitMessage(&hm,  HPI_OBJ_OSTREAM,
-				HPI_OSTREAM_HOSTBUFFER_ALLOC);
-	else
-		HPI_InitMessage(&hm,  HPI_OBJ_ISTREAM,
+	HPI_InitMessage(&hm,  obj,
+			obj == HPI_OBJ_OSTREAM ?
+				HPI_OSTREAM_HOSTBUFFER_ALLOC :
 				HPI_ISTREAM_HOSTBUFFER_ALLOC);
 
 	HPI_HandleToIndexes(hStream, &hm.wAdapterIndex,
@@ -212,16 +215,15 @@ static u16 HPI_StreamHostBufferDetach(
 {
 	struct hpi_message hm;
 	struct hpi_response hr;
+	unsigned int obj = HPI_HandleObject(hStream);
 
 	if (!hStream)
 		return HPI_ERROR_INVALID_OBJ;
 
-	if (HPI_HandleObject(hStream) ==  HPI_OBJ_OSTREAM)
-		HPI_InitMessage(&hm,  HPI_OBJ_OSTREAM,
-					HPI_OSTREAM_HOSTBUFFER_FREE);
-	else
-		HPI_InitMessage(&hm,  HPI_OBJ_ISTREAM,
-					HPI_ISTREAM_HOSTBUFFER_FREE);
+	HPI_InitMessage(&hm,  obj,
+			obj == HPI_OBJ_OSTREAM ?
+				HPI_OSTREAM_HOSTBUFFER_FREE :
+				HPI_ISTREAM_HOSTBUFFER_FREE);
 
 	HPI_HandleToIndexes(hStream, &hm.wAdapterIndex,
 				&hm.u.d.wStreamIndex, NULL);
@@ -320,7 +322,6 @@ static void print_hwparams(struct snd_pcm_hw_params *p)
 	snd_printd("period_size %d \n", params_period_size(p));
 	snd_printd("periods %d \n", params_periods(p));
 	snd_printd("buffer_size %d \n", params_buffer_size(p));
-	snd_printd("tick_time %d \n", params_tick_time(p));
 }
 #else
 #define print_hwparams(x)
@@ -394,12 +395,18 @@ static void snd_card_asihpi_pcm_samplerates(struct snd_card_asihpi *asihpi,
 		err = HPI_MixerGetControl(phSubSys, asihpi->hMixer,
 					  HPI_SOURCENODE_CLOCK_SOURCE, 0, 0, 0,
 					  HPI_CONTROL_SAMPLECLOCK, &hControl);
+		if (err) {
+			printk(KERN_ERR "No local sampleclock, err %d\n", err);
+		}
 
 		for (idx = 0; idx < 100; idx++) {
-			if (HPI_ControlQuery(phSubSys, hControl,
-					HPI_SAMPLECLOCK_SAMPLERATE,
-					idx, 0, &sampleRate))
+			if (HPI_SampleClock_QueryLocalRate(phSubSys, hControl,
+					idx, &sampleRate)) {
+				if (!idx)
+					printk(KERN_ERR "Local rate query failed\n");
+
 				break;
+			}
 
 			rate_min = min(rate_min, sampleRate);
 			rate_max = max(rate_max, sampleRate);
@@ -476,9 +483,9 @@ static int snd_card_asihpi_pcm_hw_params(struct snd_pcm_substream *substream,
 	if (err)
 		return err;
 
-	VPRINTK1(KERN_INFO "Format %d chan, %d format, %dHz\n",
-				params_channels(params),
-				wFormat, params_rate(params));
+	VPRINTK1(KERN_INFO "Format %d, %d chans, %dHz\n",
+				wFormat,params_channels(params),
+				params_rate(params));
 
 	HPI_HandleError(HPI_FormatCreate(&dpcm->Format, params_channels(params),
 			     wFormat, params_rate(params), 0, 0));
@@ -499,7 +506,9 @@ static int snd_card_asihpi_pcm_hw_params(struct snd_pcm_substream *substream,
 			params_buffer_bytes(params),  runtime->dma_addr);
 		if (err == 0) {
 			snd_printd(KERN_INFO
-					"StreamHostBufferAttach succeeded\n");
+				   "StreamHostBufferAttach succeeded %u %lu\n",
+				   params_buffer_bytes(params),
+				   (unsigned long)runtime->dma_addr);
 		} else {
 			snd_printd(KERN_INFO
 					"StreamHostBufferAttach error %d\n",
@@ -523,16 +532,8 @@ static int snd_card_asihpi_pcm_hw_params(struct snd_pcm_substream *substream,
 	dpcm->bytes_per_sec = bytes_per_sec;
 	dpcm->pcm_size = params_buffer_bytes(params);
 	dpcm->pcm_count = params_period_bytes(params);
-	snd_printd(KERN_INFO "pcm_size x%x, pcm_count x%x\n",
-			dpcm->pcm_size, dpcm->pcm_count);
-
-#ifdef FORCE_TIMER_JIFFIES
-	if (dpcm->pcm_jiffie_per_period > FORCE_TIMER_JIFFIES) {
-		dpcm->pcm_jiffie_per_period = FORCE_TIMER_JIFFIES;
-		snd_printd(KERN_INFO "Forced timer jiffies to %d\n",
-				FORCE_TIMER_JIFFIES);
-	}
-#endif
+	snd_printd(KERN_INFO "pcm_size x%x, pcm_count x%x, Bps %d\n",
+			dpcm->pcm_size, dpcm->pcm_count, bytes_per_sec);
 
 	dpcm->pcm_irq_pos = 0;
 	dpcm->pcm_buf_pos = 0;
@@ -682,8 +683,6 @@ static void snd_card_asihpi_runtime_free(struct snd_pcm_runtime *runtime)
 	kfree(dpcm);
 }
 
-/***************************** PLAYBACK OPS ****************/
-
 /*algorithm outline
  Without linking degenerates to getting single stream pos etc
  Without mmap 2nd loop degenerates to snd_pcm_period_elapsed
@@ -759,6 +758,7 @@ static void snd_card_asihpi_timer_function(unsigned long data)
 			snd_printd(KERN_WARNING  "OStream %d drained\n",
 					s->number);
 			snd_pcm_stop(s, SNDRV_PCM_STATE_XRUN);
+			return;
 		}
 
 		if (s->stream == SNDRV_PCM_STREAM_PLAYBACK) {
@@ -812,9 +812,10 @@ static void snd_card_asihpi_timer_function(unsigned long data)
 							ds->pcm_count);
 					HPI_HandleError(
 						HPI_OutStreamWriteBuf(
-							phSubSys,
-							ds->hStream,
-							NULL, xfercount,
+							phSubSys, ds->hStream,
+							&s->runtime->
+								dma_area[0],
+							xfercount,
 							&ds->Format));
 				} else {
 					VPRINTK2("Read IS%d x%04x\n",
@@ -834,6 +835,7 @@ static void snd_card_asihpi_timer_function(unsigned long data)
 		add_timer(&dpcm->timer);
 }
 
+/***************************** PLAYBACK OPS ****************/
 static int snd_card_asihpi_playback_ioctl(struct snd_pcm_substream *substream,
 					  unsigned int cmd, void *arg)
 {
@@ -841,8 +843,6 @@ static int snd_card_asihpi_playback_ioctl(struct snd_pcm_substream *substream,
 	return snd_pcm_lib_ioctl(substream, cmd, arg);
 }
 
-
-
 static int snd_card_asihpi_playback_prepare(struct snd_pcm_substream *
 					    substream)
 {
@@ -966,6 +966,7 @@ static int snd_card_asihpi_playback_open(struct snd_pcm_substream *substream)
 	runtime->private_data = dpcm;
 	runtime->private_free = snd_card_asihpi_runtime_free;
 
+	snd_card_asihpi_playback.channels_max = card->out_max_chans;
 	snd_card_asihpi_playback_format(card, dpcm->hStream,
 					&snd_card_asihpi_playback);
 
@@ -981,10 +982,11 @@ static int snd_card_asihpi_playback_open(struct snd_pcm_substream *substream)
 	if (card->support_grouping)
 		snd_card_asihpi_playback.info |= SNDRV_PCM_INFO_SYNC_START;
 
-	/* struct copy so can create initializer dynamically */
+	/* struct is copied, so can create initializer dynamically */
 	runtime->hw = snd_card_asihpi_playback;
-	/* Strictly only necessary for HPI6205 adapters */
-	err = snd_pcm_hw_constraint_pow2(runtime, 0,
+
+	if (card->support_mmap)
+		err = snd_pcm_hw_constraint_pow2(runtime, 0,
 					SNDRV_PCM_HW_PARAM_BUFFER_BYTES);
 	if (err < 0)
 		return err;
@@ -1188,9 +1190,9 @@ static int snd_card_asihpi_capture_open(struct snd_pcm_substream *substream)
 	runtime->private_data = dpcm;
 	runtime->private_free = snd_card_asihpi_runtime_free;
 
+	snd_card_asihpi_capture.channels_max = card->in_max_chans;
 	snd_card_asihpi_capture_format(card, dpcm->hStream,
 				       &snd_card_asihpi_capture);
-
 	snd_card_asihpi_pcm_samplerates(card,  &snd_card_asihpi_capture);
 	snd_card_asihpi_capture.info = SNDRV_PCM_INFO_INTERLEAVED;
 
@@ -1200,8 +1202,8 @@ static int snd_card_asihpi_capture_open(struct snd_pcm_substream *substream)
 
 	runtime->hw = snd_card_asihpi_capture;
 
-	/* Strictly only necessary for HPI6205 adapters */
-	err = snd_pcm_hw_constraint_pow2(runtime, 0,
+	if (card->support_mmap)
+		err = snd_pcm_hw_constraint_pow2(runtime, 0,
 					SNDRV_PCM_HW_PARAM_BUFFER_BYTES);
 	if (err < 0)
 		return err;
@@ -1311,6 +1313,7 @@ struct hpi_control {
 	u16 wDstNodeType;
 	u16 wDstNodeIndex;
 	u16 wBand;
+	char name[44]; /* copied to snd_ctl_elem_id.name[44]; */
 };
 
 static char *asihpi_tuner_band_names[] =
@@ -1346,6 +1349,7 @@ static char *asihpi_src_names[] =
 	"Bitstr",
 	"Mic",
 	"Cobranet",
+	"AnalogIn",
 };
 #else
 static char *asihpi_src_names[] =
@@ -1360,10 +1364,13 @@ static char *asihpi_src_names[] =
 	"Bitstream",
 	"Mic",
 	"Cobranet in",
+	"Analog in",
 };
 #endif
 
-#define NUM_SOURCENODE_STRINGS 10
+#if ((11 != (HPI_SOURCENODE_LAST_INDEX-HPI_SOURCENODE_BASE+1)))
+#error "SourceNode strings don't agree with HPI defines - version mismatch?"
+#endif
 
 #if ASI_STYLE_NAMES
 static char *asihpi_dst_names[] =
@@ -1375,6 +1382,7 @@ static char *asihpi_dst_names[] =
 	"RF",
 	"Speaker" ,
 	"Cobranet",
+	"AnalogOut",
 };
 #else
 static char *asihpi_dst_names[] =
@@ -1386,17 +1394,14 @@ static char *asihpi_dst_names[] =
 	"RF",
 	"Speaker",
 	"Cobranet out",
+	"Analog out",
 };
 #endif
-#define NUM_DESTNODE_STRINGS 7
 
-#if ((NUM_SOURCENODE_STRINGS != \
-	(HPI_SOURCENODE_LAST_INDEX-HPI_SOURCENODE_BASE+1)) || \
-      (NUM_DESTNODE_STRINGS  != (HPI_DESTNODE_LAST_INDEX-HPI_DESTNODE_BASE+1)))
-#error "Node strings don't agree with HPI defines - version mismatch?"
+#if ((8 != (HPI_DESTNODE_LAST_INDEX-HPI_DESTNODE_BASE+1)))
+#error "DestNode strings don't agree with HPI defines - version mismatch?"
 #endif
 
-static char snd_control_name[44]; /*asound.h:745 unsigned char name[44]; */
 
 static inline int ctl_add(struct snd_card *card, struct snd_kcontrol_new *ctl,
 				struct snd_card_asihpi *asihpi)
@@ -1418,25 +1423,25 @@ static void asihpi_ctl_init(struct snd_kcontrol_new *snd_control,
 				char *name)
 {
 	memset(snd_control, 0, sizeof(*snd_control));
-	snd_control->name = snd_control_name;
+	snd_control->name = asihpi_control->name;
 	snd_control->private_value = asihpi_control->hControl;
 	snd_control->iface = SNDRV_CTL_ELEM_IFACE_MIXER;
 	snd_control->index = 0;
 
 	if (asihpi_control->wSrcNodeType && asihpi_control->wDstNodeType)
-		sprintf(snd_control->name, "%s%d to %s%d %s",
+		sprintf(asihpi_control->name, "%s%d to %s%d %s",
 			asihpi_src_names[asihpi_control->wSrcNodeType],
 			asihpi_control->wSrcNodeIndex,
 			asihpi_dst_names[asihpi_control->wDstNodeType],
 			asihpi_control->wDstNodeIndex,
 			name);
 	else if (asihpi_control->wDstNodeType) {
-		sprintf(snd_control->name, "%s%d %s",
+		sprintf(asihpi_control->name, "%s%d %s",
 		asihpi_dst_names[asihpi_control->wDstNodeType],
 		asihpi_control->wDstNodeIndex,
 		name);
 	} else {
-		sprintf(snd_control->name, "%s%d %s",
+		sprintf(asihpi_control->name, "%s%d %s",
 		asihpi_src_names[asihpi_control->wSrcNodeType],
 		asihpi_control->wSrcNodeIndex,
 		name);
@@ -1530,10 +1535,26 @@ static int __devinit snd_asihpi_volume_add(struct snd_card_asihpi *asihpi,
 static int snd_asihpi_level_info(struct snd_kcontrol *kcontrol,
 				 struct snd_ctl_elem_info *uinfo)
 {
+	u32 hControl = kcontrol->private_value;
+	u16 err;
+	short nMinGain_01dB;
+	short nMaxGain_01dB;
+	short nStepGain_01dB;
+
+	err =
+	    HPI_LevelQueryRange(phSubSys, hControl, &nMinGain_01dB,
+			       &nMaxGain_01dB, &nStepGain_01dB);
+	if (err) {
+		nMaxGain_01dB = 2400;
+		nMinGain_01dB = -1000;
+		nStepGain_01dB = 100;
+	}
+
 	uinfo->type = SNDRV_CTL_ELEM_TYPE_INTEGER;
 	uinfo->count = 2;
-	uinfo->value.integer.min = -10;
-	uinfo->value.integer.max = 24;
+	uinfo->value.integer.min = nMinGain_01dB / HPI_UNITS_PER_dB;
+	uinfo->value.integer.max = nMaxGain_01dB / HPI_UNITS_PER_dB;
+	uinfo->value.integer.step = nStepGain_01dB / HPI_UNITS_PER_dB;
 	return 0;
 }
 
@@ -1763,11 +1784,10 @@ static int snd_asihpi_tuner_gain_info(struct snd_kcontrol *kcontrol,
 	u32 hControl = kcontrol->private_value;
 	u16 err;
 	short idx;
-	u32 gainRange[3];
+	u16 gainRange[3];
 
 	for (idx = 0; idx < 3; idx++) {
-	err = HPI_ControlQuery(phSubSys, hControl, HPI_TUNER_GAIN,
-				idx, 0, &gainRange[idx]);
+	err = HPI_Tuner_QueryGain(phSubSys, hControl, idx, &gainRange[idx]);
 		if (err != 0)
 			return err;
 	}
@@ -1813,42 +1833,50 @@ static int snd_asihpi_tuner_gain_put(struct snd_kcontrol *kcontrol,
 /* Band  */
 
 static int asihpi_tuner_band_query(struct snd_kcontrol *kcontrol,
-					u32 *bandList, u32 len) {
+					u16 *bandList, u32 len) {
 	u32 hControl = kcontrol->private_value;
-	u16 err;
-	u32 idx;
+	u16 err = 0;
+	u32 i;
 
-	for (idx = 0; idx < len; idx++) {
-		err = HPI_ControlQuery(phSubSys, hControl , HPI_TUNER_BAND,
-				idx, 0 , &bandList[idx]);
+	for (i = 0; i < len; i++) {
+		err = HPI_Tuner_QueryBand(phSubSys, hControl, i, &bandList[i]);
 		if (err != 0)
 			break;
 	}
 
-	return idx;
+	if (err && (err != HPI_ERROR_INVALID_OBJ_INDEX))
+		return -EIO;
+
+	return i;
 }
 
 static int snd_asihpi_tuner_band_info(struct snd_kcontrol *kcontrol,
 				  struct snd_ctl_elem_info *uinfo)
 {
-	u32 tunerBands[HPI_TUNER_BAND_LAST];
-	u32 numBands = 0;
+	u16 tunerBands[HPI_TUNER_BAND_LAST];
+	int numBands = 0;
 
 	numBands = asihpi_tuner_band_query(kcontrol, tunerBands,
 				HPI_TUNER_BAND_LAST);
 
+	if (numBands < 0)
+		return numBands;
+
 	uinfo->type = SNDRV_CTL_ELEM_TYPE_ENUMERATED;
 	uinfo->count = 1;
 	uinfo->value.enumerated.items = numBands;
 
-	if (uinfo->value.enumerated.item >= uinfo->value.enumerated.items)
-		uinfo->value.enumerated.item =
-			uinfo->value.enumerated.items - 1;
+	if (numBands > 0) {
+		if (uinfo->value.enumerated.item >=
+					uinfo->value.enumerated.items)
+			uinfo->value.enumerated.item =
+				uinfo->value.enumerated.items - 1;
 
-	strcpy(uinfo->value.enumerated.name,
-		asihpi_tuner_band_names[
+		strcpy(uinfo->value.enumerated.name,
+			asihpi_tuner_band_names[
 				tunerBands[uinfo->value.enumerated.item]]);
 
+	}
 	return 0;
 }
 
@@ -1860,7 +1888,7 @@ static int snd_asihpi_tuner_band_get(struct snd_kcontrol *kcontrol,
 	struct snd_card_asihpi *asihpi = snd_kcontrol_chip(kcontrol);
 	*/
 	u16 band, idx;
-	u32 tunerBands[HPI_TUNER_BAND_LAST];
+	u16 tunerBands[HPI_TUNER_BAND_LAST];
 	u32 numBands = 0;
 
 	numBands = asihpi_tuner_band_query(kcontrol, tunerBands,
@@ -1886,7 +1914,7 @@ static int snd_asihpi_tuner_band_put(struct snd_kcontrol *kcontrol,
 	*/
 	u32 hControl = kcontrol->private_value;
 	u16 band;
-	u32 tunerBands[HPI_TUNER_BAND_LAST];
+	u16 tunerBands[HPI_TUNER_BAND_LAST];
 	u32 numBands = 0;
 
 	numBands = asihpi_tuner_band_query(kcontrol, tunerBands,
@@ -1905,8 +1933,8 @@ static int snd_asihpi_tuner_freq_info(struct snd_kcontrol *kcontrol,
 {
 	u32 hControl = kcontrol->private_value;
 	u16 err;
-	u32 tunerBands[HPI_TUNER_BAND_LAST];
-	u32 numBands = 0, band_iter, idx;
+	u16 tunerBands[HPI_TUNER_BAND_LAST];
+	u16 numBands = 0, band_iter, idx;
 	u32 freqRange[3], tempFreqRange[3];
 
 	numBands = asihpi_tuner_band_query(kcontrol, tunerBands,
@@ -1918,8 +1946,8 @@ static int snd_asihpi_tuner_freq_info(struct snd_kcontrol *kcontrol,
 
 	for (band_iter = 0; band_iter < numBands; band_iter++) {
 		for (idx = 0; idx < 3; idx++) {
-			err = HPI_ControlQuery(phSubSys, hControl,
-				HPI_TUNER_FREQ, idx, tunerBands[band_iter],
+			err = HPI_Tuner_QueryFrequency(phSubSys, hControl,
+				idx, tunerBands[band_iter],
 				&tempFreqRange[idx]);
 			if (err != 0)
 				return err;
@@ -2107,7 +2135,7 @@ static int snd_asihpi_meter_get(struct snd_kcontrol *kcontrol,
 }
 
 static int __devinit snd_asihpi_meter_add(struct snd_card_asihpi *asihpi,
-					struct hpi_control *asihpi_control)
+					struct hpi_control *asihpi_control, int subidx)
 {
 	struct snd_card *card = asihpi->card;
 	struct snd_kcontrol_new snd_control;
@@ -2118,7 +2146,9 @@ static int __devinit snd_asihpi_meter_add(struct snd_card_asihpi *asihpi,
 	snd_control.info = snd_asihpi_meter_info;
 	snd_control.get = snd_asihpi_meter_get;
 
-	return (ctl_add(card, &snd_control, asihpi));
+	snd_control.index = subidx;
+
+	return ctl_add(card, &snd_control, asihpi);
 }
 
 /*------------------------------------------------------------
@@ -2242,15 +2272,14 @@ static int snd_asihpi_cmode_info(struct snd_kcontrol *kcontrol,
 	};
 
 	u32 hControl = kcontrol->private_value;
-	u32 mode;
+	u16 wMode;
 	int i;
 
 	/* HPI channel mode values can be from 1 to 6
 	Some adapters only support a contiguous subset
 	*/
 	for (i = 0; i < HPI_CHANNEL_MODE_LAST; i++)
-		if (HPI_ControlQuery(phSubSys, hControl,
-				HPI_CHANNEL_MODE_MODE, i, 0, &mode))
+		if (HPI_ChannelMode_QueryMode(phSubSys,  hControl, i, &wMode))
 			break;
 
 	uinfo->type = SNDRV_CTL_ELEM_TYPE_ENUMERATED;
@@ -2464,15 +2493,15 @@ static int __devinit snd_asihpi_sampleclock_add(struct snd_card_asihpi *asihpi,
 	u32 hSC =  asihpi_control->hControl;
 	int hasAesIn = 0;
 	int i, j;
-	u32 wSource;
+	u16 wSource;
 
 	snd_control.private_value = asihpi_control->hControl;
 
 	clkcache->has_local = 0;
 
 	for (i = 0; i <= HPI_SAMPLECLOCK_SOURCE_LAST; i++) {
-		if  (HPI_ControlQuery(phSubSys, hSC,
-				HPI_SAMPLECLOCK_SOURCE, i, 0, &wSource))
+		if  (HPI_SampleClock_QuerySource(phSubSys, hSC,
+				i, &wSource))
 			break;
 		clkcache->s[i].source = wSource;
 		clkcache->s[i].index = 0;
@@ -2485,9 +2514,9 @@ static int __devinit snd_asihpi_sampleclock_add(struct snd_card_asihpi *asihpi,
 	if (hasAesIn)
 		/* already will have picked up index 0 above */
 		for (j = 1; j < 8; j++) {
-			if (HPI_ControlQuery(phSubSys, hSC,
-				HPI_SAMPLECLOCK_SOURCE_INDEX, j,
-				HPI_SAMPLECLOCK_SOURCE_AESEBU_INPUT, &wSource))
+			if (HPI_SampleClock_QuerySourceIndex(phSubSys, hSC,
+				j, HPI_SAMPLECLOCK_SOURCE_AESEBU_INPUT,
+				&wSource))
 				break;
 			clkcache->s[i].source =
 				HPI_SAMPLECLOCK_SOURCE_AESEBU_INPUT;
@@ -2535,10 +2564,12 @@ static int __devinit snd_card_asihpi_mixer_new(struct snd_card_asihpi *asihpi)
 {
 	struct snd_card *card = asihpi->card;
 	unsigned int idx = 0;
+	unsigned int subindex = 0;
 	int err;
-	struct hpi_control asihpi_control;
+	struct hpi_control asihpi_control, prev_control;
 
-	snd_assert(asihpi != NULL, return -EINVAL);
+	if (snd_BUG_ON(!asihpi))
+		return -EINVAL;
 	strcpy(card->mixername, "Asihpi Mixer");
 
 	err =
@@ -2572,6 +2603,21 @@ static int __devinit snd_card_asihpi_mixer_new(struct snd_card_asihpi *asihpi)
 		asihpi_control.wSrcNodeType -= HPI_SOURCENODE_BASE;
 		asihpi_control.wDstNodeType -= HPI_DESTNODE_BASE;
 
+		/* ASI50xx in SSX mode has multiple meters on the same node.
+		   Use subindex to create distinct ALSA controls
+		   for any duplicated controls.
+		*/
+		if ((asihpi_control.wControlType == prev_control.wControlType) &&
+		    (asihpi_control.wSrcNodeType == prev_control.wSrcNodeType) &&
+		    (asihpi_control.wSrcNodeIndex == prev_control.wSrcNodeIndex) &&
+		    (asihpi_control.wDstNodeType == prev_control.wDstNodeType) &&
+		    (asihpi_control.wDstNodeIndex == prev_control.wDstNodeIndex))
+			subindex++;
+		else
+			subindex = 0;
+
+		prev_control = asihpi_control;
+
 		switch (asihpi_control.wControlType) {
 		case HPI_CONTROL_VOLUME:
 			err = snd_asihpi_volume_add(asihpi, &asihpi_control);
@@ -2586,10 +2632,11 @@ static int __devinit snd_card_asihpi_mixer_new(struct snd_card_asihpi *asihpi)
 			err = snd_asihpi_cmode_add(asihpi, &asihpi_control);
 			break;
 		case HPI_CONTROL_METER:
-			err = snd_asihpi_meter_add(asihpi, &asihpi_control);
+			err = snd_asihpi_meter_add(asihpi, &asihpi_control, subindex);
 			break;
 		case HPI_CONTROL_SAMPLECLOCK:
-			err = snd_asihpi_sampleclock_add(asihpi, &asihpi_control);
+			err = snd_asihpi_sampleclock_add(
+						asihpi, &asihpi_control);
 			break;
 		case HPI_CONTROL_CONNECTION:	/* ignore these */
 			continue;
@@ -2744,7 +2791,7 @@ static int __devinit snd_asihpi_hpi_new(struct snd_card_asihpi *asihpi,
 /*------------------------------------------------------------
    CARD
  ------------------------------------------------------------*/
-int __devinit snd_asihpi_probe(struct pci_dev *pci_dev,
+static int __devinit snd_asihpi_probe(struct pci_dev *pci_dev,
 				       const struct pci_device_id *pci_id)
 {
 	int err;
@@ -2776,17 +2823,19 @@ int __devinit snd_asihpi_probe(struct pci_dev *pci_dev,
 	hpi_card = pci_get_drvdata(pci_dev);
 
 	/* first try to give the card the same index as its hardware index */
-	card = snd_card_new(hpi_card->index,
-			id[hpi_card->index], THIS_MODULE,
-			sizeof(struct snd_card_asihpi));
-	if (card == NULL) {
+	err = snd_card_create(hpi_card->index,
+			      id[hpi_card->index], THIS_MODULE,
+			      sizeof(struct snd_card_asihpi),
+			      &card);
+	if (err < 0) {
 		/* if that fails, try the default index==next available */
-		card =
-		    snd_card_new(index[dev], id[dev],
-				THIS_MODULE,
-				sizeof(struct snd_card_asihpi));
-		if (card == NULL)
-			return -ENOMEM;
+		err =
+		    snd_card_create(index[dev], id[dev],
+				    THIS_MODULE,
+				    sizeof(struct snd_card_asihpi),
+				    &card);
+		if (err < 0)
+			return err;
 		snd_printk(KERN_WARNING
 			"**** WARNING **** Adapter index %d->ALSA index %d\n",
 			hpi_card->index, card->number);
@@ -2831,8 +2880,19 @@ int __devinit snd_asihpi_probe(struct pci_dev *pci_dev,
 			((asihpi->wType & 0xF000) == 0x6000));
 
 
-	printk(KERN_INFO "Supports mmap:%d grouping:%d\n",
-			asihpi->support_mmap, asihpi->support_grouping);
+	err = HPI_AdapterGetProperty(phSubSys, asihpi->wAdapterIndex,
+		HPI_ADAPTER_PROPERTY_CURCHANNELS,
+		&asihpi->in_max_chans, &asihpi->out_max_chans);
+	if (err) {
+		asihpi->in_max_chans = 2;
+		asihpi->out_max_chans = 2;
+	}
+
+	printk(KERN_INFO "Supports mmap:%d grouping:%d mrx:%d\n",
+			asihpi->support_mmap,
+			asihpi->support_grouping,
+			asihpi->support_mrx
+	      );
 
 	HPI_InStreamClose(phSubSys, hStream);
 
@@ -2886,7 +2946,7 @@ static void __devexit snd_asihpi_remove(struct pci_dev *pci_dev)
 	asihpi_adapter_remove(pci_dev);
 }
 
-struct pci_device_id  asihpi_pci_tbl[] = {
+static struct pci_device_id  asihpi_pci_tbl[] = {
 	{HPI_PCI_VENDOR_ID_TI, HPI_ADAPTER_DSP6205,
 		HPI_PCI_VENDOR_ID_AUDIOSCIENCE, PCI_ANY_ID, 0, 0,
 		(kernel_ulong_t)HPI_6205},
